      program dumpinterp
 
c...  This program reads a TERRA dump for one grid resolution and number
c...  of processors and generates another TERRA dump for a higher grid
c...  resolution and larger number of processors.
 
      include 'size.h'
      include 'pcom.h'
 
      parameter (nr1= 64, mt1=128, nt1=16, kt1=nt*mt1/mt)
 
      real temp0((nt+1)**2*nd*(nr+1)), u0((nt+1)**2*nd*3*(nr+1))
      real pres0((nt+1)**2*nd*(nr+1)), pmap0((nt+1)**2*nd*2)
      real temp1((kt1+1)**2*nd*(nr1+1)), u1((kt1+1)**2*nd*3*(nr1+1))
      real pres1((kt1+1)**2*nd*(nr1+1)), pmap1((kt1+1)**2*nd*2)
      real temp2((nt1+1)**2*nd*(nr1+1)), u2((nt1+1)**2*nd*3*(nr1+1))
      real pres2((nt1+1)**2*nd*(nr1+1)), pmap2((nt1+1)**2*nd*2)
      real w((kt1+1)**2*nd*3*(nr1+1)), y((kt1+1)**2*nd*3*(nr1+1)/4)
      real rshl(129)
      common /prty/ propr(20)
      common /name/ titl(4,4)
      character titl*8, cname*12, fname*12, casenum0*3, casenum1*3
      character char1*1, char2*1, char3*1, char4*1, char5*1, char6*1
 
      npedg  = mt/nt
      npedg1 = mt1/nt1
      nproc  = npedg**2*10/nd
      nlvl   = 1.45*log(real(mt1/mt))
      iblk   = npedg1/npedg
 
      CALL MPI_INIT( IERROR )
      CALL MPI_COMM_RANK( MPI_COMM_WORLD, MYNUM, IERROR )
      CALL MPI_COMM_SIZE( MPI_COMM_WORLD, NPROC, IERROR )
 
      casenum0 = '211'
      idump0   =   7
      casenum1 = '221'
      idump1   =  01
 
      iproc = mynum
 
      char1 = char(48 + iproc/1000)
      char2 = char(48 + mod(iproc/100,10))
      char3 = char(48 + mod(iproc/10,10))
      char4 = char(48 + mod(iproc,10))
      char5 = char(48 + idump0/10)
      char6 = char(48 + mod(idump0,10))
      cname = 'c'//casenum0//'.'//char1//char2//char3//char4
     &                     //'.'//char5//char6
 
      open (7, file=cname, status='unknown')
 
c...  Read in temperature, velocity, pressure, and plate map fields.
 
      call vecin(temp0,titl,rshl,1,nr,7,1)
      call vecin(u0   ,titl,rshl,3,nr,7,0)
      call vecin(pres0,titl,rshl,1,nr,7,0)
 
      if(propr(20) .gt. 0) call vecin(pmap0,titl,rshl,1,1,7,1)
 
c...  Interpolate temperature field to the finer grid.
 
      nrf = min(nr1, nr*2)
      ntf = nt*2
 
      call interp3s(w,temp0,1,nd,nr,nrf,ntf)
 
      do lv=2,nlvl
         call scopy((ntf+1)**2*nd*(nrf+1), w, 1, y, 1)
         nrc = nrf
         nrf = min(nr1, nrc*2)
         ntf = ntf*2
         call interp3s(w,y,1,nd,nrc,nrf,ntf)
      end do
 
      call scopy((kt1+1)**2*nd*(nr1+1), w, 1, temp1, 1)
 
c...  Interpolate velocity field to the finer grid.
 
      nrf = min(nr1, nr*2)
      ntf = nt*2
 
      call interp3s(w,u0,3,nd,nr,nrf,ntf)
 
      do lv=2,nlvl
         call scopy((ntf+1)**2*nd*3*(nrf+1), w, 1, y, 1)
         nrc = nrf
         nrf = min(nr1, nrc*2)
         ntf = ntf*2
         call interp3s(w,y,3,nd,nrc,nrf,ntf)
      end do
 
      call scopy((kt1+1)**2*nd*3*(nr1+1), w, 1, u1, 1)
 
c...  Interpolate pressure field to the finer grid.
 
      nrf = min(nr1, nr*2)
      ntf = nt*2
 
      call interp3s(w,pres0,1,nd,nr,nrf,ntf)
 
      do lv=2,nlvl
         call scopy((ntf+1)**2*nd*(nrf+1), w, 1, y, 1)
         nrc = nrf
         nrf = min(nr1, nrc*2)
         ntf = ntf*2
         call interp3s(w,y,1,nd,nrc,nrf,ntf)
      end do
 
      call scopy((kt1+1)**2*nd*(nr1+1), w, 1, pres1, 1)
 
c...  Interpolate plate map field, if present, to the finer grid.
 
      if(propr(20) .gt. 0) then
 
         ntf = nt*2
 
         call interpmap(w,pmap0,nd,2,ntf)
 
         do lv=2,nlvl
            call scopy((ntf+1)**2*nd*2, w, 1, y, 1)
            ntf = ntf*2
            call interpmap(w,y,nd,2,ntf)
         end do
 
         call scopy((kt1+1)**2*nd*2, w, 1, pmap1, 1)
 
      end if
 
      iblock = -1
 
      do jb=0,iblk-1
         do ib=0,iblk-1
 
            iblock = iblock + 1
 
c...        Split interpolated fields into subdomain blocks.
 
            call domainsplit(temp2,temp1,nd*(nr1+1),nt1,kt1,iblock)
            call domainsplit(u2   ,u1 ,3*nd*(nr1+1),nt1,kt1,iblock)
            call domainsplit(pres2,pres1,nd*(nr1+1),nt1,kt1,iblock)
            if(propr(20) .gt. 0.)
     &         call domainsplit(pmap2,pmap1,nd*2   ,nt1,kt1,iblock)
 
c...        Write temperature, velocity, pressure, and plate map
c...        fields to output file.
 
            jproc = mod(iproc,npedg)*iblk + ib
     &            + (iproc/npedg)*npedg1*iblk + jb*npedg1
 
            char1 = char(48 + jproc/1000)
            char2 = char(48 + mod(jproc/100,10))
            char3 = char(48 + mod(jproc/10,10))
            char4 = char(48 + mod(jproc,10))
            char5 = char(48 + idump1/10)
            char6 = char(48 + mod(idump1,10))
            fname = 'c'//casenum1//'.'//char1//char2//char3//char4
     &                           //'.'//char5//char6
 
            open (8, file=fname, status='unknown')
 
            call vecout(temp2,rshl,1,nr1,nt1,nr1,8,1)
            call vecout(u2   ,rshl,3,nr1,nt1,nr1,8,0)
            call vecout(pres2,rshl,1,nr1,nt1,nr1,8,0)
            if(propr(20) .gt. 0.)
     &         call vecout(pmap2,rshl,1,nr1,nt1, 1,8,1)
 
            close (8)
 
         end do
      end do
 
      close (7)
 
      CALL MPI_FINALIZE( IERROR )
 
      end
*dk interp3s
      subroutine interp3s(zf,zc,nj,nd,nrc,nrf,ntf)
 
c...  This routine performs a linear interpolation of the nodal
c...  field zc defined on a coarse mesh to yield the field zf
c...  defined on a mesh one dyadic level finer.
 
      real zf((ntf+1)**2,*), zc((ntf/2+1)**2,*)
 
      ntfp = ntf + 1
      ntc  = ntf/2
      ntcp = ntc + 1
      njnd = nj*nd
 
c...  Interpolate in lateral directions.
 
      do i2=1,ntc
         ii0 = (i2 - 1)*ntcp
         jj0 = (i2 + i2 - 2)*ntfp - 1
         do i1=2,ntcp
            ii = ii0 + i1
            jj = jj0 + i1 + i1
            do kk=1,njnd*(nrc+1)
               zf(jj     ,kk) =      zc(ii,kk)
               zf(jj-1   ,kk) = 0.5*(zc(ii,kk) + zc(ii-1   ,kk))
               zf(jj+ntf ,kk) = 0.5*(zc(ii,kk) + zc(ii+ntc ,kk))
               zf(jj+ntfp,kk) = 0.5*(zc(ii,kk) + zc(ii+ntcp,kk))
            end do
         end do
      end do
 
c...  Treat upper right edge of diamond.
 
      do i2=1,ntc
         ii = (i2 - 1)*ntcp + 1
         jj = (i2 + i2 - 2)*ntfp + 1
         do kk=1,njnd*(nrc+1)
            zf(jj     ,kk) =      zc(ii,kk)
            zf(jj+ntfp,kk) = 0.5*(zc(ii,kk) + zc(ii+ntcp,kk))
         end do
      end do
 
c...  Treat lower right edge of diamond.
 
      do i1=2,ntcp
         ii = ntc*ntcp + i1
         jj = ntf*ntfp + i1 + i1 - 1
         do kk=1,njnd*(nrc+1)
            zf(jj     ,kk) =      zc(ii,kk)
            zf(jj-1   ,kk) = 0.5*(zc(ii,kk) + zc(ii-1   ,kk))
         end do
      end do
 
c...  Treat right corner of diamond.
 
      ii = ntc*ntcp + 1
      jj = ntf*ntfp + 1
      do kk=1,njnd*(nrc+1)
         zf(jj     ,kk) =      zc(ii,kk)
      end do
 
      if(nrf .gt. nrc) then
 
c...     Interpolate in radial direction.
 
         do ir=nrc,1,-1
            do kk=1,njnd
               jr = ir*njnd   + kk
               lr = ir*njnd*2 + kk
               kr = lr - njnd
               do jj=1,ntfp*ntfp
                  zf(jj,lr) =      zf(jj,jr)
                  zf(jj,kr) = 0.5*(zf(jj,jr) + zf(jj,jr-njnd))
               end do
            end do
         end do
 
      endif
 
      end
*dk interpmap
      subroutine interpmap(pmf,pmc,nd,nlr,ntf)
 
c...  This routine performs an interpolation of the nodal map
c...  field pmc defined on a coarse mesh to yield the field pmf
c...  defined on a mesh one dyadic level finer.
 
      real pmf((ntf+1)**2,*), pmc((ntf/2+1)**2,*)
 
      ntfp = ntf + 1
      ntc  = ntf/2
      ntcp = ntc + 1
 
c...  Interpolate in lateral directions.
 
      do i2=1,ntc
         ii0 = (i2 - 1)*ntcp
         jj0 = (i2 + i2 - 2)*ntfp - 1
         do i1=2,ntcp
            ii = ii0 + i1
            jj = jj0 + i1 + i1
            do kk=1,nd*nlr
               pmf(jj     ,kk) =     pmc(ii,kk)
               pmf(jj-1   ,kk) = min(pmc(ii,kk), pmc(ii-1   ,kk))
               pmf(jj+ntf ,kk) = min(pmc(ii,kk), pmc(ii+ntc ,kk))
               pmf(jj+ntfp,kk) = min(pmc(ii,kk), pmc(ii+ntcp,kk))
            end do
         end do
      end do
 
c...  Treat upper right edge of diamond.
 
      do i2=1,ntc
         ii = (i2 - 1)*ntcp + 1
         jj = (i2 + i2 - 2)*ntfp + 1
         do kk=1,nd*nlr
            pmf(jj     ,kk) =     pmc(ii,kk)
            pmf(jj+ntfp,kk) = min(pmc(ii,kk), pmc(ii+ntcp,kk))
         end do
      end do
 
c...  Treat lower right edge of diamond.
 
      do i1=2,ntcp
         ii = ntc*ntcp + i1
         jj = ntf*ntfp + i1 + i1 - 1
         do kk=1,nd*nlr
            pmf(jj     ,kk) =     pmc(ii,kk)
            pmf(jj-1   ,kk) = min(pmc(ii,kk), pmc(ii-1   ,kk))
         end do
      end do
 
c...  Treat right corner of diamond.
 
      ii = ntc*ntcp + 1
      jj = ntf*ntfp + 1
      do kk=1,nd*nlr
         pmf(jj     ,kk) = pmc(ii,kk)
      end do
 
      end
*dk domainsplit
      subroutine domainsplit(sp,s,nc,nt,kt,iblock)
 
c...  This routine reads the appropriate portion of the input array s
c...  into the output sub-array sp corresponding to block iblock.
 
      real sp(0:nt,nt+1,nc), s(0:kt,kt+1,nc)
 
c...  Determine subdomain limits.
 
      rnm   = real(nt)/real(kt)
      j0    = iblock*rnm
      i0    = (real(iblock)*rnm - j0)/rnm
 
      ibeg  = i0*nt
      jbeg  = j0*nt + 1
      iend  = ibeg  + nt
      jend  = jbeg  + nt
 
c...  Load sub-array.
 
      do kk=1,nc
 
         i  = -1
 
         do ii=ibeg,iend
 
            i =  i + 1
            j = 0
 
            do jj=jbeg,jend
 
               j = j + 1
 
               sp(i,j,kk) = s(ii,jj,kk)
 
            end do
 
         end do
 
      end do
 
      end
*dk vecin
      subroutine vecin(u,titl,rshl,nj,mr,nf,ifmt)
 
c...  This routine reads the nodal field u from logical unit nf
c...  using 1pe10.3 format when ifmt = 0 and f10.3 when ifmt = 1.
 
      include 'size.h'
      real u(*), rshl(*)
      common /prty/ propr(20)
      character*8 titl(4,4)
 
      read(nf,10) kr, kt
 10   format(2i5)
 
      if(kr.ne.nr .or. kt.ne.nt) then
         print *,'Array dimensions of data file and size.h inconsistent'
         stop
      end if
 
      read(nf,20) titl
 20   format(4a8)
 
      read(nf,30) (rshl(i),i=1,nr+1)
      read(nf,30) propr
 30   format(1p10e15.8)
 
      kr = mr
      if(mr .lt. 0) kr = nr
 
      if(ifmt .eq. 0) then
 
         read(nf,40) (u(ii),ii=1,(nt+1)**2*nd*nj*(kr+1))
 40      format(1p15e10.3)
 
      elseif(ifmt .eq. 1) then
 
         read(nf,50) (u(ii),ii=1,(nt+1)**2*nd*nj*(kr+1))
 50      format(15f10.3)
 
      endif
 
      end
*dk vecout
      subroutine vecout(u,rshl,nj,kr,kt,mr,nf,ifmt)
 
c...  This routine writes the nodal field u to logical unit nf
c...  using 1pe10.3 format when ifmt = 0 and f10.3 when ifmt = 1.
 
      include 'size.h'
      real u(*), rshl(*)
      common /name/ titl(4,4)
      common /prty/ propr(20)
      character*8 titl
 
      write(nf,10) kr, kt
 10   format(2i5)
 
      write(nf,20) titl
 20   format(4a8)
 
      write(nf,30) (rshl(i),i=1,kr+1)
      write(nf,30) propr
 30   format(1p10e15.8)
 
      if(ifmt .eq. 0) then
 
         write(nf,40) (u(ii),ii=1,(kt+1)**2*nd*nj*(mr+1))
 40      format(1p15e10.3)
 
      elseif(ifmt .eq. 1) then
 
         write(nf,50) (u(ii),ii=1,(kt+1)**2*nd*nj*(mr+1))
 50      format(15f10.3)
 
      endif
 
      end
