*DK advect
      subroutine advect(dhdt)
 
c...  This routine computes the heat advected and conducted per unit
c...  time and also accounts for phase transitions if they are present.
 
      include 'size.h'
      real dhdt((nt+1)**2,nd,*), dh((nt+1)**2,2)
      common /velo/ upb(nt+1), u((nt+1)**2,nd,3,nr+1),  upe(nt+1)
      common /temp/ tpb(nt+1), temp((nt+1)**2,nd,nr+1), tpe(nt+1)
      common /mesh/ xn((nt+1)**2,nd,3)
      common /radl/ rshl(nr+1), ird, ibc
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /face/   psi((nt+1)**2,3), rdxn((nt+1)**2,3),
     &              facet((nt+1)**2,nd,3,3)
      common /ndar/  arn((nt+1)**2),  arne((nt+1)**2),
     &              rarn((nt+1)**2), rarne((nt+1)**2)
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /eos1/ rhorf(129), tmprf(129), drhdp(129), grv(129)
      common /eos2/ rhocv(129), gamma(129), alpha(129), prf(129),
     &               cond(129)
      common /cycl/ iter, time, tstep, step
      common /qdfc/ j1f(4), j2f(4), jef(4), i1n(3), i2n(3)
      common /clck/ itmng, sec(50), nop(50)
      if(itmng.eq.1) call second(tin)
 
      call nulvec(dhdt, (nt+1)**2*nd*(nr+1))
 
      i410 = 0
      i660 = 0
 
      do ir=1,nr
 
         jr = ir + 1
         rm = 0.5*(rshl(ir) + rshl(jr))
         r2 = rm**2
         if(jr .le. nr) ra = r2 - 0.25*(rshl(jr) + rshl(jr+1))**2
         if(jr .gt. nr) ra = r2 - rshl(nr+1)**2
         cr = 0.5*(cond(ir) + cond(jr))*rshl(ir)*rshl(jr)
     &          /(rshl(ir) - rshl(jr))
         ct = ra*cond(jr)/rshl(jr)
         ai = (gamma(ir) - 1.)*rhocv(ir)
         aj = (gamma(jr) - 1.)*rhocv(jr)
         if(i410.eq.0 .and. rshl(1)-rshl(jr).gt.410.e3) i410 = ir
         if(i660.eq.0 .and. rshl(1)-rshl(jr).gt.660.e3) i660 = ir
 
         do id=1,nd
 
c...        Add contributions from radial advection and conduction,
c...        adiabatic heating, and phase changes.
 
            ii1 = 2
            if(mod(id,5) .eq. 1) ii1 = 1
 
            do ii=ii1,(nt+1)**2
               uf1  = 0.5*(u(ii,id,1,ir) + u(ii,id,1,jr))
               uf2  = 0.5*(u(ii,id,2,ir) + u(ii,id,2,jr))
               uf3  = 0.5*(u(ii,id,3,ir) + u(ii,id,3,jr))
               vfl  = ((uf1*xn(ii,id,1) + uf2*xn(ii,id,2))
     &                + uf3*xn(ii,id,3))*r2*arn(ii)
               tadv = 0.5*(temp(ii,id,ir) + temp(ii,id,jr))*vfl
               cnd  = (temp(ii,id,jr) - temp(ii,id,ir))*cr*arn(ii)
               dh(ii,1) = vfl
               dhdt(ii,id,ir) = (dhdt(ii,id,ir) + tadv*rhocv(ir)) + cnd
     &                        +  temp(ii,id,ir)*vfl*ai
               dhdt(ii,id,jr) = (dhdt(ii,id,jr) - tadv*rhocv(jr)) - cnd
     &                        -  temp(ii,id,jr)*vfl*aj
            end do
 
c...        Add latent heat from phase transition if appropriate.
 
            if(ieos.ge.10 .and. (ir.eq.i410 .or. ir.eq.i660)) then
 
               if(ir.eq.i410) then
                  cl =  cl410*230./3610.
                  bb = 0.60*1.4e10*230./3610.
                  if(ieos .ne. 10) bb = 0.
                  at = (rshl(1) - 410.e3 - rshl(i410+1))
     &                      /(rshl(i410) - rshl(i410+1))
               else
                  cl =  cl660*380./4185.
                  bb = 0.60*2.4e10*380./4185.
                  if(ieos .ne. 10) bb = 0.
                  at = (rshl(1) - 660.e3 - rshl(i660+1))
     &                      /(rshl(i660) - rshl(i660+1))
               end if
 
               ab = 1. - at
               do ii=ii1,(nt+1)**2
                  aa = (cl*(at*temp(ii,id,ir) + ab*temp(ii,id,jr))
     &               -  bb)*dh(ii,1)
                  dhdt(ii,id,ir) = dhdt(ii,id,ir) - aa*at
                  dhdt(ii,id,jr) = dhdt(ii,id,jr) - aa*ab
               end do
 
            end if
 
c...        Add contributions from lateral advection and conduction.
 
            do ifc=1,3
 
               jj = i1n(ifc) + (nt+1)*i2n(ifc)
               ib = max(1, 1-jj)
               ie = min((nt+1)**2, (nt+1)**2-jj)
 
               do ii=ib,ie
                  uf1  = 0.5*(u(ii,id,1,jr) + u(ii+jj,id,1,jr))
                  uf2  = 0.5*(u(ii,id,2,jr) + u(ii+jj,id,2,jr))
                  uf3  = 0.5*(u(ii,id,3,jr) + u(ii+jj,id,3,jr))
                  vfl  = ((uf1*facet(ii,id,ifc,1)
     &                   + uf2*facet(ii,id,ifc,2))
     &                   + uf3*facet(ii,id,ifc,3))*psi(ii,ifc)*ra
                  hadv = 0.5*(temp(ii,id,jr) + temp(ii+jj,id,jr))
     &                      *vfl*rhocv(jr)
                  cnd  = ct*(temp(ii,id,jr) - temp(ii+jj,id,jr))
     &                  *psi(ii,ifc)*rdxn(ii,ifc)
                  dh(ii,1) = hadv + cnd
                  dh(ii,2) = vfl*aj
                  dhdt(ii,id,jr) = dhdt(ii,id,jr) - dh(ii,1)
     &                           - temp(ii,id,jr) * dh(ii,2)
               end do
 
               do ii=ib,ie
                  dhdt(ii+jj,id,jr) = dhdt(ii+jj,id,jr) + dh(ii,1)
     &                              + temp(ii+jj,id,jr) * dh(ii,2)
               end do
 
            end do
 
         end do
 
      end do
 
      call comm3s(dhdt,nr,nt,1)
 
c...  Divide by nodal volume.
 
      do ir=1,nr+1
         do id=1,nd
            do ii=1,(nt+1)**2
               dhdt(ii,id,ir) = dhdt(ii,id,ir)*vol(ii,ir,2)
            end do
         end do
      end do
 
      if(itmng.eq.1) call second(tout)
      if(itmng.eq.1) sec(21) = sec(21) + tout - tin
      end
*dk energy
      subroutine energy(dhdt)
 
c...  This routine updates the temperature field.
 
      include 'size.h'
      include 'pcom.h'
      real dhdt((nt+1)**2,nd,nr+1)
      common /flds/ f((nt+1)**2,nd,nr+1,8)
      common /temp/ tpb(nt+1), temp((nt+1)**2,nd,nr+1), tpe(nt+1)
      common /shht/ shb(nt+1),  shr((nt+1)**2,nd,nr+1), she(nt+1)
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /heat/ htop, hbot, hrad, heat, hshr, hvol, hnet, tnrm,
     &              tav(129), qc(128)
      common /eos1/ rhorf(129), tmprf(129), drhdp(129), grv(129)
      common /eos2/ rhocv(129), gamma(129), alpha(129), prf(129),
     &               cond(129)
      common /moon/ imoon, radis(5), tcore, qcgn, visc0
      common /volc/ tmpmlt
      common /call/ ncall
      common /clck/ itmng, sec(50), nop(50)
      if(itmng.eq.1) call second(tin)
 
      if(ncall .eq. 1) call facegen(f)
 
c...  Compute the rate of temperature change due to advection and
c...  conduction.
 
      call advect(dhdt)
 
c...  Add volume heating contribution.
 
      if(hgen .gt. 0.) then
         do ir=1,nr+1
            do id=1,nd
               do ii=1,(nt+1)**2
                  dhdt(ii,id,ir) = dhdt(ii,id,ir) + hgen*rhorf(ir)
               end do
            end do
         end do
      end if
 
c...  Add shear heating contribution.
 
      if(ieos .ge. 10) then
         do ir=1,nr+1
            do id=1,nd
               do ii=1,(nt+1)**2
                  dhdt(ii,id,ir) = dhdt(ii,id,ir) + shr(ii,id,ir)
               end do
            end do
         end do
      end if
 
      htop = 0.
      hbot = 0.
 
c...  Add cooling due to melting and magma migration.
 
      if(tmpmlt .ne. 0.) call volcano(dhdt,temp)
 
c...  Use heating rate of the nodal boundary layers as heat flux
c...  leaving or entering the shell.
 
      do id=1,nd
 
         do ii=1,(nt+1)**2
            htop = htop + dhdt(ii,id,1)*vol(ii,1,1)
            dhdt(ii,id,1) = 0.
         end do
 
         if(tb(2) .ne. 0.) then
            do ii=1,(nt+1)**2
               hbot = hbot - dhdt(ii,id,nr+1)*vol(ii,nr+1,1)
               dhdt(ii,id,nr+1) = 0.
            end do
         end if
 
      end do
 
      if(nproc .gt. 1) call psum(htop, 1)
      if(nproc .gt. 1) call psum(hbot, 1)
 
c...  Convert from heating rate to rate of change in temperature.
 
      do ir=2,nr+1
         a1 = 1./rhocv(ir)
         do id=1,nd
            do ii=1,(nt+1)**2
               dhdt(ii,id,ir) = a1*dhdt(ii,id,ir)
            end do
         end do
      end do
 
      if(imoon.eq.1 .and. tb(2).ne.0.) then
         do id=1,nd
            do ii=1,(nt+1)**2
               temp(ii,id,nr+1) = tb(2)
            end do
         end do
      end if
 
      if(itmng.eq.1) call second(tout)
      if(itmng.eq.1) sec(20) = sec(20) + tout - tin
      end
*dk facegen
      subroutine facegen(xc)
 
c...  This routine computes the arrays facet, psi, and rdxn.  Array
c...  facet contains the unit normals to the quadrilateral faces; psi
c...  contains the half-angle subtended by the faces with respect to the
c...  center of the sphere, and rdxn contains the reciprocal distances
c...  between the nodes on opposite sides of the faces.  Three faces
c...  are associated with each node, and these are numbered in a
c...  counter-clockwise order for diamonds linked with the north pole.
c...  Since these quantities are identical for all faces in a given
c...  radial stack, they are computed only for one layer.
 
      include 'size.h'
      include 'pcom.h'
      real xc(nt+1,0:nt,2,3)
      common /xnex/ xne(-1:nt+1,0:nt+2,nd,3)
      common /face/   psi(0:nt,nt+1,3), rdxn(0:nt,nt+1,3),
     &              facet(0:nt,nt+1,nd,3,3)
      common /qdfc/ j1f(4), j2f(4), jef(4), i1n(3), i2n(3)
 
      call nulvec(facet,(nt+1)**2*nd*9)
      call nulvec(psi  ,(nt+1)**2*3)
      call nulvec(rdxn ,(nt+1)**2*3)
 
      id0 = (mynum/mproc)*nd
 
      do id=1,nd
 
      jd  = id + id0
 
      call tricntr(xc,xne,id)
 
      do ifc=1,3
 
         je = jef(ifc)
         ke = jef(ifc+1)
 
         do i2=1,nt
 
            j2 = i2 + j2f(ifc)
            k2 = i2 + j2f(ifc+1)
            m2 = i2 + i2n(ifc)
 
            do i1=1,nt
 
               j1 = i1 + j1f(ifc)
               k1 = i1 + j1f(ifc+1)
               m1 = i1 + i1n(ifc)
 
               if(id .eq. 1) then
 
c...              Compute the reciprocal distance between nodes.
 
                  aa = (xne(i1,i2,1,1) - xne(m1,m2,1,1))**2
     &               + (xne(i1,i2,1,2) - xne(m1,m2,1,2))**2
     &               + (xne(i1,i2,1,3) - xne(m1,m2,1,3))**2
                  rdxn(i1,i2,ifc) = 1./sqrt(aa)
 
c...              Compute the half-angle psi between the two points
c...              defining the face with respect to the center of
c...              the sphere.
 
                  aa = (xc(k1,k2,ke,1) - xc(j1,j2,je,1))**2
     &               + (xc(k1,k2,ke,2) - xc(j1,j2,je,2))**2
     &               + (xc(k1,k2,ke,3) - xc(j1,j2,je,3))**2
                  psi(i1,i2,ifc) = asin(0.5*sqrt(aa))
 
               endif
 
c...           The array facet is given by the cross product of vectors
c...           from the origin to the two points defining the face,
c...           normalized to unity and pointing outward from the center
c...           of the cell.
 
               aa = 1.
               if(jd .ge. 6) aa = -1.
               aa = aa/sin(2.*psi(i1,i2,ifc))
 
               facet(i1,i2,id,ifc,1) =
     &                             aa*(xc(k1,k2,ke,2)*xc(j1,j2,je,3)
     &                               - xc(k1,k2,ke,3)*xc(j1,j2,je,2))
               facet(i1,i2,id,ifc,2) =
     &                             aa*(xc(k1,k2,ke,3)*xc(j1,j2,je,1)
     &                               - xc(k1,k2,ke,1)*xc(j1,j2,je,3))
               facet(i1,i2,id,ifc,3) =
     &                             aa*(xc(k1,k2,ke,1)*xc(j1,j2,je,2)
     &                               - xc(k1,k2,ke,2)*xc(j1,j2,je,1))
 
            end do
 
         end do
 
      end do
 
      end do
 
      end
*dk heating
      subroutine heating(heat,hrad,tdot)
 
c...  This routine computes the total heating rate from the time rate
c...  of change of temperature tdot as well as the radiogenic heating
c...  rate.
 
      include 'size.h'
      include 'pcom.h'
      real tdot((nt+1)**2,nd,nr+1)
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /radl/ rshl(nr+1), ird, ibc
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /eos1/ rhorf(129), tmprf(129), drhdp(129), grv(129)
      common /eos2/ rhocv(129), gamma(129), alpha(129), prf(129),
     &               cond(129)
 
      heat = 0.
      davg = 0.
      svol = 0.
 
      do ir=1,nr+1
 
         do id=1,nd
            do ii=1,(nt+1)**2
               heat = heat + rhocv(ir)*tdot(ii,id,ir)*vol(ii,ir,1)
            end do
         end do
 
         davg = davg + vol(2,ir,1)*rhorf(ir)
         svol = svol + vol(2,ir,1)
 
      end do
 
      hrad = 4.18879*hgen*davg/svol*(rshl(1)**3 - rshl(nr+1)**3)
 
      if(nproc .gt. 1) call psum(heat,1)
 
      end
*dk shrheat
      subroutine shrheat(dhdt,u,z,g)
 
c...  This routine computes the shear heating rate.
 
      include 'size.h'
      include 'pcom.h'
      real dhdt((nt+1)**2,nd,*), u((nt+1)**2,nd,3,*)
      real    g((nt+1)**2,nd,3), z((nt+1)**2,nd,3,3), rr(3,2,129)
      common /grad/ rg(3,2,nr+1), grd(7,(nt+1)**2,3,2)
      common /ndar/  arn((nt+1)**2),  arne((nt+1)**2),
     &              rarn((nt+1)**2), rarne((nt+1)**2)
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /ofst/ j1n(7), j2n(7), md(7)
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /heat/ htop, hbot, hrad, heat, hshr, hvol, hnet, tnrm,
     &              tav(129), qc(128)
      common /eos2/ rhocv(129), gamma(129), alpha(129), prf(129),
     &               cond(129)
      common /vis0/ vb(nt+1), vv((nt+1)**2*nd*(nr+1),2), ve(nt+1)
      common /vis1/ vscmax, rvscscl, tvscscl, pwrlawn, pwrlawsr, yldstrs
      common /shrh/ shrlayr(129)
      common /call/ ncall
      common /clck/ itmng, sec(50), nop(50)
      if(itmng.eq.1) call second(tin)
 
      twothd = 2./3.
 
      call nulvec(dhdt,(nt+1)**2*nd*(nr+1))
 
      aa = sqrt(visc)
      do 10  k=1,3
      do 10 ir=1,nr+1
      rr(k,1,ir) = aa*rg(k,1,ir)
      rr(k,2,ir) = aa*rg(k,2,ir)
 10   continue
 
      do 80 ir=1,nr+1
 
c...  Compute the node-to-node gradient of velocity.
 
      call nulvec(z,(nt+1)**2*nd*9)
 
      k1 = 1
      k2 = 3
      if(ir .eq.    1) k1 = 2
      if(ir .eq. nr+1) k2 = 2
 
      do 60  k=k1,k2
      do 60  m=1,7
 
      jj  = j1n(m) + (nt+1)*j2n(m)
      jr  = ir + k - 2
 
      do 20 ii=1,(nt+1)**2
      g(ii,1,1) = vol(ii,ir,2)*(grd(m,ii,1,1)*rr(k,1,ir)
     &                        + grd(m,ii,1,2)*rr(k,2,ir))
      g(ii,1,2) = vol(ii,ir,2)*(grd(m,ii,2,1)*rr(k,1,ir)
     &                        + grd(m,ii,2,2)*rr(k,2,ir))
      g(ii,1,3) = vol(ii,ir,2)*(grd(m,ii,3,1)*rr(k,1,ir)
     &                        + grd(m,ii,3,2)*rr(k,2,ir))
 20   continue
 
      do 30 id=2,nd
      do 30 ii=1,(nt+1)**2
      g(ii,id,1) = g(ii,1,1)
      g(ii,id,2) = g(ii,1,2)
      g(ii,id,3) = g(ii,1,3)
 30   continue
 
      call rotate(g,nd,0,nt,-1)
 
      do 60 id=1,nd
 
      do 40 ii=1,(nt+1)**2
      z(ii,id,1,1) = z(ii,id,1,1) + g(ii,id,1)*u(ii+jj,id,1,jr)
      z(ii,id,2,1) = z(ii,id,2,1) + g(ii,id,2)*u(ii+jj,id,1,jr)
      z(ii,id,3,1) = z(ii,id,3,1) + g(ii,id,3)*u(ii+jj,id,1,jr)
      z(ii,id,1,2) = z(ii,id,1,2) + g(ii,id,1)*u(ii+jj,id,2,jr)
      z(ii,id,2,2) = z(ii,id,2,2) + g(ii,id,2)*u(ii+jj,id,2,jr)
      z(ii,id,3,2) = z(ii,id,3,2) + g(ii,id,3)*u(ii+jj,id,2,jr)
 40   continue
 
      do 50 ii=1,(nt+1)**2
      z(ii,id,1,3) = z(ii,id,1,3) + g(ii,id,1)*u(ii+jj,id,3,jr)
      z(ii,id,2,3) = z(ii,id,2,3) + g(ii,id,2)*u(ii+jj,id,3,jr)
      z(ii,id,3,3) = z(ii,id,3,3) + g(ii,id,3)*u(ii+jj,id,3,jr)
 50   continue
 
 60   continue
 
      call comm3s(z,2,nt,3)
 
      do 70 id=1,nd
      do 70 ii=1,(nt+1)**2
      dhdt(ii,id,ir) = (((2.*((z(ii,id,1,1)**2
     &   +  z(ii,id,2,2)**2) + z(ii,id,3,3)**2)
     &   + (z(ii,id,1,2) + z(ii,id,2,1))**2)
     &   + (z(ii,id,2,3) + z(ii,id,3,2))**2)
     &   + (z(ii,id,3,1) + z(ii,id,1,3))**2)
     &   - (z(ii,id,1,1) + z(ii,id,2,2) + z(ii,id,3,3))**2*twothd
 70   continue
 
 80   continue
 
      if(vscmax .gt. 0.) then
 
         do ir=1,nr+1
            do id=1,nd
               jj = (nd*(ir - 1) + (id - 1))*(nt + 1)**2
               do ii=1,(nt+1)**2
                  dhdt(ii,id,ir) = dhdt(ii,id,ir)*vv(ii+jj,1)**2
               end do
            end do
         end do
 
      end if
 
      hshr = 0.
      do 130 ir=1,nr+1
      shrlayr(ir) = 0.
      do 120 id=1,nd
      do 120 ii=1,(nt+1)**2
      dh   = dhdt(ii,id,ir)*vol(ii,ir,1)
      hshr = hshr + dh
      shrlayr(ir) = shrlayr(ir) + dh
 120  continue
      shrlayr(ir) = shrlayr(ir)*arn(2)/(4.*3.14159265*vol(2,ir,1))
 130  continue
 
      if(nproc .gt. 1) call psum(hshr,1)
      if(nproc .gt. 1) call psum(shrlayr,nr+1)
 
      if(itmng.eq.1) call second(tout)
      if(itmng.eq.1) sec(22) = sec(22) + 0.25*(tout - tin)
      end
*dk volcano
      subroutine volcano(dhdt,temp)
 
c...  This routine adds cooling to reduce the temperature to the
c...  melting temperature where this temperature is exceeded.  This
c...  is intended to model melting and rapid heat removal by magma
c...  migration to the surface.
 
      include 'size.h'
      include 'pcom.h'
      real dhdt((nt+1)**2,nd,nr+1), temp((nt+1)**2,nd,nr+1)
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /radl/ rshl(nr+1), ird, ibc
      common /cycl/ iter, time, tstep, step
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /heat/ htop, hbot, hrad, heat, hshr, hvol, hnet, tnrm,
     &              tav(129), qc(128)
      common /eos2/ rhocv(129), gamma(129), alpha(129), prf(129),
     &               cond(129)
      common /volc/ tmpmlt
 
      if(tmpmlt .eq. 0.) return
 
      hvol = 0.
 
      irmx = nr/3
      if(tb(2).eq.0. .and. ieos.eq.1) irmx = nr
 
      do ir=2,irmx
 
         aa   = 0.
         if(tstep .gt. 0.) aa = rhocv(ir)/tstep
         tmlt = tmpmlt + 1.8e-3*(rshl(1) - rshl(ir))
         if(ir .gt. nr/4) tmlt = min(tmlt, 3000.)
 
         do id=1,nd
            do ii=1,(nt+1)**2
               cool = max(0., aa*(temp(ii,id,ir) - tmlt))
               hvol = hvol + cool*vol(ii,ir,1)
               dhdt(ii,id,ir) = dhdt(ii,id,ir) - cool
            end do
         end do
 
      end do
 
      htop = htop + hvol
 
      if(nproc .gt. 1) call psum(hvol, 1)
 
      end
*dk bdenergy
      block data bdenergy
 
      common /qdfc/ j1f(4), j2f(4), jef(4), i1n(3), i2n(3)
      data j1f/1,0,0,0/, j2f/0,0,0,-1/, jef/1,2,1,2/
      data i1n/0,-1,-1/, i2n/1,1,0/
 
      end
