*DK divergence
      subroutine divergence(d,t,u,v,q,r,iflag)
 
c...  This routine computes the divergence d of various possible
c...  scalar products with the velocity field u, depending on the
c...  value of the flag iflag as follows:
c...     When iflag = 0, the routine simply computes div(u).
c...     When iflag = 1, the routine computes
c...                     div[(rhorf/rho)*u]/(rhorf/rho)**2
c...     When iflag = 2, the routine computes div(t*u).
c...  The second case involves weighting the velocity by the normalized
c...  radial density variation needed in the anelastic approximation.
c...  The last case corresponds to advection of the scalar field t by
c...  the velocity field u.
 
c...  The routine applies the transpose of the finite element gradient
c...  operator to obtain the divergence.  Since the finite element
c...  operator contains a factor of volume, it is necessary to solve
c...  the linear system  M*d = D*(s*u) for d, where M is the finite
c...  element mass matrix, D is the transpose of the gradient, and s
c...  is the appropriate field multiplying the velocity u.  Jacobi
c...  iteration is used to solve this well-conditioned system.  The
c...  arrays v, q, and r are work arrays.
 
      include 'size.h'
      real d((nt+1)**2,nd,nr+1), v((nt+1)**2,nd,3,nr+1)
      real t((nt+1)**2,nd,nr+1), r((nt+1)**2,nd,  nr+1)
      real q((nt+1)**2,nd,nr+1), w(2,(nt+1)**2,nr+1,nd)
      common /grad/ rg(3,2,nr+1), grd(7,(nt+1)**2,3,2)
      common /eos1/ rhorf(129), tmprf(129), drhdp(129), grv(129)
      common /prop/ ieos,  rho0, visc,  grav,  texpn, tcond,
     &              sheat, hgen, tb(2), cl410, cl660
      common /volm/ vol((nt+1)**2,nr+1,2)
      common /radl/ rshl(nr+1), ird, ibc
      common /clck/ itmng, sec(50), nop(50)
 
      if(itmng.eq.1) call second(tin)
 
      nt1 = nt + 1

      call scopy((nt+1)**2*(nr+1)*3*nd,u,1,v,1)
      call rotate(v,nd,nr,nt,1)
 
      if(iflag .eq. 1) then
 
         do ir=1,nr+1
            a1 = rhorf(ir)/rho0
            do id=1,nd
               do ii=1,(nt+1)**2
                  v(ii,id,1,ir) = a1*v(ii,id,1,ir)
                  v(ii,id,2,ir) = a1*v(ii,id,2,ir)
                  v(ii,id,3,ir) = a1*v(ii,id,3,ir)
               end do
            end do
         end do
 
      elseif(iflag .eq. 2) then
 
         do ir=1,nr+1
            do id=1,nd
               do ii=1,(nt+1)**2
                  v(ii,id,1,ir) = t(ii,id,ir)*v(ii,id,1,ir)
                  v(ii,id,2,ir) = t(ii,id,ir)*v(ii,id,2,ir)
                  v(ii,id,3,ir) = t(ii,id,ir)*v(ii,id,3,ir)
               end do
            end do
         end do
 
      elseif(iflag .ne. 0) then
 
         print *, 'STOP--invalid value for iflag in routine divergence.'
         stop
 
      end if
 
        call lsetv(w,1,2*(nt+1)**2*(nr+1)*nd,0.0)
 
      do j=1,3
!DIR$ NOINTERCHANGE
        do id=1,nd
 
!DIR$ NOINTERCHANGE
          do ir=1,nr,2
            do ii = 1,(nt+1)**2
                w(1,ii,ir,id) = w(1,ii,ir,id) + (((((((
     &              grd(1,ii,j,1)*v(ii    ,id,j,ir))
     &            + grd(2,ii,j,1)*v(ii+1  ,id,j,ir))
     &            + grd(3,ii,j,1)*v(ii+nt1,id,j,ir))
     &            + grd(4,ii,j,1)*v(ii+nt ,id,j,ir))
     &            + grd(5,ii,j,1)*v(ii-1  ,id,j,ir))
     &            + grd(6,ii,j,1)*v(ii-nt1,id,j,ir))
     &            + grd(7,ii,j,1)*v(ii-nt ,id,j,ir))
                w(1,ii,ir+1,id) = w(1,ii,ir+1,id) + (((((((
     &              grd(1,ii,j,1)*v(ii    ,id,j,ir+1))
     &            + grd(2,ii,j,1)*v(ii+1  ,id,j,ir+1))
     &            + grd(3,ii,j,1)*v(ii+nt1,id,j,ir+1))
     &            + grd(4,ii,j,1)*v(ii+nt ,id,j,ir+1))
     &            + grd(5,ii,j,1)*v(ii-1  ,id,j,ir+1))
     &            + grd(6,ii,j,1)*v(ii-nt1,id,j,ir+1))
     &            + grd(7,ii,j,1)*v(ii-nt ,id,j,ir+1))
                w(2,ii,ir,id) = w(2,ii,ir,id) + (((((((
     &              grd(1,ii,j,2)*v(ii    ,id,j,ir))
     &            + grd(2,ii,j,2)*v(ii+1  ,id,j,ir))
     &            + grd(3,ii,j,2)*v(ii+nt1,id,j,ir))
     &            + grd(4,ii,j,2)*v(ii+nt ,id,j,ir))
     &            + grd(5,ii,j,2)*v(ii-1  ,id,j,ir))
     &            + grd(6,ii,j,2)*v(ii-nt1,id,j,ir))
     &            + grd(7,ii,j,2)*v(ii-nt ,id,j,ir))
                w(2,ii,ir+1,id) = w(2,ii,ir+1,id) + (((((((
     &              grd(1,ii,j,2)*v(ii    ,id,j,ir+1))
     &            + grd(2,ii,j,2)*v(ii+1  ,id,j,ir+1))
     &            + grd(3,ii,j,2)*v(ii+nt1,id,j,ir+1))
     &            + grd(4,ii,j,2)*v(ii+nt ,id,j,ir+1))
     &            + grd(5,ii,j,2)*v(ii-1  ,id,j,ir+1))
     &            + grd(6,ii,j,2)*v(ii-nt1,id,j,ir+1))
     &            + grd(7,ii,j,2)*v(ii-nt ,id,j,ir+1))
            end do
          end do
          do ii = 1,(nt+1)**2
              w(1,ii,nr+1,id) = w(1,ii,nr+1,id) + (((((((
     &            grd(1,ii,j,1)*v(ii    ,id,j,nr+1))
     &          + grd(2,ii,j,1)*v(ii+1  ,id,j,nr+1))
     &          + grd(3,ii,j,1)*v(ii+nt1,id,j,nr+1))
     &          + grd(4,ii,j,1)*v(ii+nt ,id,j,nr+1))
     &          + grd(5,ii,j,1)*v(ii-1  ,id,j,nr+1))
     &          + grd(6,ii,j,1)*v(ii-nt1,id,j,nr+1))
     &          + grd(7,ii,j,1)*v(ii-nt ,id,j,nr+1))
              w(2,ii,nr+1,id) = w(2,ii,nr+1,id) + (((((((
     &            grd(1,ii,j,2)*v(ii    ,id,j,nr+1))
     &          + grd(2,ii,j,2)*v(ii+1  ,id,j,nr+1))
     &          + grd(3,ii,j,2)*v(ii+nt1,id,j,nr+1))
     &          + grd(4,ii,j,2)*v(ii+nt ,id,j,nr+1))
     &          + grd(5,ii,j,2)*v(ii-1  ,id,j,nr+1))
     &          + grd(6,ii,j,2)*v(ii-nt1,id,j,nr+1))
     &          + grd(7,ii,j,2)*v(ii-nt ,id,j,nr+1))
          end do
        end do
 
      end do
 
      do id=1,nd
 
        do ii=1,(nt+1)**2
            q(ii,id,1) =
     &        (((rg(2,1,1)*w(1,ii,1,id)
     &         + rg(2,2,1)*w(2,ii,1,id))
     &         + rg(3,1,1)*w(1,ii,2,id))
     &         + rg(3,2,1)*w(2,ii,2,id))
            q(ii,id,nr+1) =
     &        (((rg(1,1,nr+1)*w(1,ii,nr  ,id)
     &         + rg(1,2,nr+1)*w(2,ii,nr  ,id))
     &         + rg(2,1,nr+1)*w(1,ii,nr+1,id))
     &         + rg(2,2,nr+1)*w(2,ii,nr+1,id))
        enddo
 
        do ir=2,nr
          do ii=1,(nt+1)**2
              q(ii,id,ir) =
     &          (((((rg(1,1,ir)*w(1,ii,ir-1,id)
     &             + rg(1,2,ir)*w(2,ii,ir-1,id))
     &             + rg(2,1,ir)*w(1,ii,ir  ,id))
     &             + rg(2,2,ir)*w(2,ii,ir  ,id))
     &             + rg(3,1,ir)*w(1,ii,ir+1,id))
     &             + rg(3,2,ir)*w(2,ii,ir+1,id))
          end do
        end do
 
      end do
 
      call comm3s(q,nr,nt,1)
 
      do ir=1,nr+1
         do id=1,nd
            do ii=1,(nt+1)**2
               d(ii,id,ir) = q(ii,id,ir)*vol(ii,ir,2)
            end do
         end do
      end do
 
      itermax = 3
      if(iflag.eq.1 .and. ibc.eq.6) itermax = 0
 
      do iter=1,itermax
 
         call massmtrx(r,d)
 
         do ir=1,nr+1
            do id=1,nd
               do ii=1,(nt+1)**2
                     r(ii,id,ir) = q(ii,id,ir) - r(ii,id,ir)
                     d(ii,id,ir) = d(ii,id,ir) + r(ii,id,ir)
     &                                *1.3*vol(ii,ir,2)
               end do
            end do
         end do
 
      end do
 
      if(iflag .eq. 1) then
 
         do ir=1,nr+1
            a1 = (rho0/rhorf(ir))**2
            do id=1,nd
               do ii=1,(nt+1)**2
                  d(ii,id,ir) = a1*d(ii,id,ir)
               end do
            end do
         end do
 
      end if
 
      if(itmng.eq.1) call second(tout)
      if(itmng.eq.1) sec(17) = sec(17) + tout - tin
      end
*dk gradient
      subroutine gradient(g,s)
 
c...  This routine computes the gradient g of the nodal scalar field s.
 
      include 'size.h'
      real g((nt+1)**2,nd,3,nr+1), s((nt+1)**2,nd,nr+1)
      real w(2,(nt+1)**2,nr+1)
      common /grad/ rg(3,2,nr+1), grd(7,(nt+1)**2,3,2)
      common /clck/ itmng, sec(50), nop(50)
      if(itmng.eq.1) call second(tin)
 
      nt1 = nt + 1
      do id=1,nd
         do j=1,3
!DIR$ NOINTERCHANGE
            do ir=1,nr,2
               do ii=1,(nt+1)**2
                  w(1,ii,ir) =
     &             ((((((grd(1,ii,j,1)*s(ii    ,id,ir)
     &                 + grd(2,ii,j,1)*s(ii+1  ,id,ir))
     &                 + grd(3,ii,j,1)*s(ii+nt1,id,ir))
     &                 + grd(4,ii,j,1)*s(ii+nt ,id,ir))
     &                 + grd(5,ii,j,1)*s(ii-1  ,id,ir))
     &                 + grd(6,ii,j,1)*s(ii-nt1,id,ir))
     &                 + grd(7,ii,j,1)*s(ii-nt ,id,ir))
                  w(1,ii,ir+1) =
     &             ((((((grd(1,ii,j,1)*s(ii    ,id,ir+1)
     &                 + grd(2,ii,j,1)*s(ii+1  ,id,ir+1))
     &                 + grd(3,ii,j,1)*s(ii+nt1,id,ir+1))
     &                 + grd(4,ii,j,1)*s(ii+nt ,id,ir+1))
     &                 + grd(5,ii,j,1)*s(ii-1  ,id,ir+1))
     &                 + grd(6,ii,j,1)*s(ii-nt1,id,ir+1))
     &                 + grd(7,ii,j,1)*s(ii-nt ,id,ir+1))
                  w(2,ii,ir) =
     &             ((((((grd(1,ii,j,2)*s(ii    ,id,ir)
     &                 + grd(2,ii,j,2)*s(ii+1  ,id,ir))
     &                 + grd(3,ii,j,2)*s(ii+nt1,id,ir))
     &                 + grd(4,ii,j,2)*s(ii+nt ,id,ir))
     &                 + grd(5,ii,j,2)*s(ii-1  ,id,ir))
     &                 + grd(6,ii,j,2)*s(ii-nt1,id,ir))
     &                 + grd(7,ii,j,2)*s(ii-nt ,id,ir))
                  w(2,ii,ir+1) =
     &             ((((((grd(1,ii,j,2)*s(ii    ,id,ir+1)
     &                 + grd(2,ii,j,2)*s(ii+1  ,id,ir+1))
     &                 + grd(3,ii,j,2)*s(ii+nt1,id,ir+1))
     &                 + grd(4,ii,j,2)*s(ii+nt ,id,ir+1))
     &                 + grd(5,ii,j,2)*s(ii-1  ,id,ir+1))
     &                 + grd(6,ii,j,2)*s(ii-nt1,id,ir+1))
     &                 + grd(7,ii,j,2)*s(ii-nt ,id,ir+1))
               end do
            end do
 
            do ii=1,(nt+1)**2
               w(1,ii,nr+1) =
     &          ((((((grd(1,ii,j,1)*s(ii    ,id,nr+1)
     &              + grd(2,ii,j,1)*s(ii+1  ,id,nr+1))
     &              + grd(3,ii,j,1)*s(ii+nt1,id,nr+1))
     &              + grd(4,ii,j,1)*s(ii+nt ,id,nr+1))
     &              + grd(5,ii,j,1)*s(ii-1  ,id,nr+1))
     &              + grd(6,ii,j,1)*s(ii-nt1,id,nr+1))
     &              + grd(7,ii,j,1)*s(ii-nt ,id,nr+1))
               w(2,ii,nr+1) =
     &          ((((((grd(1,ii,j,2)*s(ii    ,id,nr+1)
     &              + grd(2,ii,j,2)*s(ii+1  ,id,nr+1))
     &              + grd(3,ii,j,2)*s(ii+nt1,id,nr+1))
     &              + grd(4,ii,j,2)*s(ii+nt ,id,nr+1))
     &              + grd(5,ii,j,2)*s(ii-1  ,id,nr+1))
     &              + grd(6,ii,j,2)*s(ii-nt1,id,nr+1))
     &              + grd(7,ii,j,2)*s(ii-nt ,id,nr+1))
            end do
 
            do ii=1,(nt+1)**2
               g(ii,id,j,1) =
     &          (((rg(2,1,1)*w(1,ii,1)
     &           + rg(2,2,1)*w(2,ii,1))
     &           + rg(3,1,1)*w(1,ii,2))
     &           + rg(3,2,1)*w(2,ii,2))
            end do
 
            do ir=2,nr
               do ii=1,(nt+1)**2
                  g(ii,id,j,ir) =
     &             (((((rg(1,1,ir)*w(1,ii,ir-1)
     &                + rg(1,2,ir)*w(2,ii,ir-1))
     &                + rg(2,1,ir)*w(1,ii,ir  ))
     &                + rg(2,2,ir)*w(2,ii,ir  ))
     &                + rg(3,1,ir)*w(1,ii,ir+1))
     &                + rg(3,2,ir)*w(2,ii,ir+1))
               end do
            end do
 
            do ii=1,(nt+1)**2
               g(ii,id,j,nr+1) =
     &          (((rg(1,1,nr+1)*w(1,ii,nr)
     &           + rg(1,2,nr+1)*w(2,ii,nr))
     &           + rg(2,1,nr+1)*w(1,ii,nr+1))
     &           + rg(2,2,nr+1)*w(2,ii,nr+1))
            end do
 
         end do
 
      end do
 
      call rotate(g,nd,nr,nt,-1)
      call comm3s(g,nr,nt,3)
 
      if(itmng.eq.1) call second(tout)
      if(itmng.eq.1) sec(19) = sec(19) + tout - tin
      end
 
*dk massmtrx
      subroutine massmtrx(v,u)
 
c...  This routine computes  v = M*u,  where M represents
c...  the mass matrix.
 
      include 'size.h'
      common /oper/ ra(3,8,nr+1), alp(7,(nt+1)**2,2),
     &              atn(4,7,(nt+1)**2,9)
      common /offset/ m(8)
      real v((nt+1)**2,nd,nr+1), u((nt+1)**2,nd,nr+1),
     &     w((nt+1)**2,nd,nr+1)
 
      nt1 = nt + 1

      do ir=1,nr+1
!DIR$ NOINTERCHANGE
        do id=1,nd
          do ii=1,(nt+1)**2
            w(ii,id,ir) = ((((((
     &                    alp(1,ii,2)*u(ii    ,id,ir)
     &                  + alp(2,ii,2)*u(ii+1  ,id,ir))
     &                  + alp(3,ii,2)*u(ii+nt1,id,ir))
     &                  + alp(4,ii,2)*u(ii+nt ,id,ir))
     &                  + alp(5,ii,2)*u(ii-1  ,id,ir))
     &                  + alp(6,ii,2)*u(ii-nt1,id,ir))
     &                  + alp(7,ii,2)*u(ii-nt ,id,ir))
          end do
        end do
      end do
 
      do id=1,nd
        do ii=1,(nt+1)**2
          v(ii,id,1)    = ra(2,7,1)*w(ii,id,1)
     &                  + ra(3,7,1)*w(ii,id,2)
          v(ii,id,nr+1) = ra(1,7,nr+1)*w(ii,id,nr)
     &                  + ra(2,7,nr+1)*w(ii,id,nr+1)
        end do
      end do
 
      do ir=2,nr
        do id=1,nd
          do ii=1,(nt+1)**2
            v(ii,id,ir) = ((ra(1,7,ir)*w(ii,id,ir-1)
     &                  + ra(2,7,ir)*w(ii,id,ir))
     &                  + ra(3,7,ir)*w(ii,id,ir+1))
          end do
        end do
      end do
 
      call comm3s(v,nr,nt,1)
 
      end
 
*dk oprxuf
      subroutine oprxuf(v,u,d)

c     This routine computes v = v - A*u, where A is a tensor operator.

      include 'size.h'
      real v((nt+1)**2,nd,3,nr+1), d((nt+1)**2,ndo,3,3,nr+1)
      real u((nt+1)**2,nd,3,nr+1), w(4,(nt+1)**2,nr+1,nd)
      common /oper/ ra(3,8,nr+1), alp(7,0:nt,nt+1,2),
     &              atn(4,7,(nt+1)**2,3,3)

      call nuledge(v,nd,nr,nt,3)

      call rotate(u,nd,nr,nt,1)
      call rotate(v,nd,nr,nt,1)

      nt1 = nt + 1
      do j=1,3

         call nulvec(w, 4*(nt+1)*(nt+1)*(nr+1)*nd)

         do i=1,3
            do ir=1,nr+1
               do id=1,nd
                  do ii=1,(nt+1)**2
                     w(1,ii,ir,id) = w(1,ii,ir,id) + (((((((
     &                  atn(1,1,ii,i,j)*u(ii    ,id,i,ir))
     &                + atn(1,2,ii,i,j)*u(ii+1  ,id,i,ir))
     &                + atn(1,3,ii,i,j)*u(ii+nt1,id,i,ir))
     &                + atn(1,4,ii,i,j)*u(ii+nt ,id,i,ir))
     &                + atn(1,5,ii,i,j)*u(ii-1  ,id,i,ir))
     &                + atn(1,6,ii,i,j)*u(ii-nt1,id,i,ir))
     &                + atn(1,7,ii,i,j)*u(ii-nt ,id,i,ir))
                     w(2,ii,ir,id) = w(2,ii,ir,id) + (((((((
     &                  atn(2,1,ii,i,j)*u(ii    ,id,i,ir))
     &                + atn(2,2,ii,i,j)*u(ii+1  ,id,i,ir))
     &                + atn(2,3,ii,i,j)*u(ii+nt1,id,i,ir))
     &                + atn(2,4,ii,i,j)*u(ii+nt ,id,i,ir))
     &                + atn(2,5,ii,i,j)*u(ii-1  ,id,i,ir))
     &                + atn(2,6,ii,i,j)*u(ii-nt1,id,i,ir))
     &                + atn(2,7,ii,i,j)*u(ii-nt ,id,i,ir))
                     w(3,ii,ir,id) = w(3,ii,ir,id) + (((((((
     &                  atn(3,1,ii,i,j)*u(ii    ,id,i,ir))
     &                + atn(3,2,ii,i,j)*u(ii+1  ,id,i,ir))
     &                + atn(3,3,ii,i,j)*u(ii+nt1,id,i,ir))
     &                + atn(3,4,ii,i,j)*u(ii+nt ,id,i,ir))
     &                + atn(3,5,ii,i,j)*u(ii-1  ,id,i,ir))
     &                + atn(3,6,ii,i,j)*u(ii-nt1,id,i,ir))
     &                + atn(3,7,ii,i,j)*u(ii-nt ,id,i,ir))
                     w(4,ii,ir,id) = w(4,ii,ir,id) + (((((((
     &                  atn(4,1,ii,i,j)*u(ii    ,id,i,ir))
     &                + atn(4,2,ii,i,j)*u(ii+1  ,id,i,ir))
     &                + atn(4,3,ii,i,j)*u(ii+nt1,id,i,ir))
     &                + atn(4,4,ii,i,j)*u(ii+nt,id,i,ir))
     &                + atn(4,5,ii,i,j)*u(ii-1  ,id,i,ir))
     &                + atn(4,6,ii,i,j)*u(ii-nt1,id,i,ir))
     &                + atn(4,7,ii,i,j)*u(ii-nt ,id,i,ir))
                     end do
                  end do
               end do
            end do

            do id=1,nd

               do ii=1,(nt+1)**2
                  v(ii,id,j,1) = v(ii,id,j,1) - ((((((((
     &              ra(2,1,1)*w(1,ii,1,id))
     &            + ra(2,2,1)*w(2,ii,1,id))
     &            + ra(2,3,1)*w(3,ii,1,id))
     &            + ra(2,4,1)*w(4,ii,1,id))
     &            + ra(3,1,1)*w(1,ii,2,id))
     &            + ra(3,2,1)*w(2,ii,2,id))
     &            + ra(3,3,1)*w(3,ii,2,id))
     &            + ra(3,4,1)*w(4,ii,2,id))
               end do

               do ir=2,nr
                  do ii=1,(nt+1)**2
                     v(ii,id,j,ir) = v(ii,id,j,ir) - ((((((((((((
     &                 ra(1,1,ir)*w(1,ii,ir-1,id))
     &               + ra(1,2,ir)*w(2,ii,ir-1,id))
     &               + ra(1,3,ir)*w(3,ii,ir-1,id))
     &               + ra(1,4,ir)*w(4,ii,ir-1,id))
     &               + ra(2,1,ir)*w(1,ii,ir  ,id))
     &               + ra(2,2,ir)*w(2,ii,ir  ,id))
     &               + ra(2,3,ir)*w(3,ii,ir  ,id))
     &               + ra(2,4,ir)*w(4,ii,ir  ,id))
     &               + ra(3,1,ir)*w(1,ii,ir+1,id))
     &               + ra(3,2,ir)*w(2,ii,ir+1,id))
     &               + ra(3,3,ir)*w(3,ii,ir+1,id))
     &               + ra(3,4,ir)*w(4,ii,ir+1,id))
                  end do
               end do

               do ii=1,(nt+1)**2
                  v(ii,id,j,nr+1) = v(ii,id,j,nr+1) - ((((((((
     &              ra(1,1,nr+1)*w(1,ii,nr,id))
     &            + ra(1,2,nr+1)*w(2,ii,nr,id))
     &            + ra(1,3,nr+1)*w(3,ii,nr,id))
     &            + ra(1,4,nr+1)*w(4,ii,nr,id))
     &            + ra(2,1,nr+1)*w(1,ii,nr+1,id))
     &            + ra(2,2,nr+1)*w(2,ii,nr+1,id))
     &            + ra(2,3,nr+1)*w(3,ii,nr+1,id))
     &            + ra(2,4,nr+1)*w(4,ii,nr+1,id))
               end do

            end do

         end do

      do ir=1,nr+1
         do id=1,nd
            jd = min(ndo, id)
            do ii=1,(nt+1)**2
               v(ii,id,1,ir) = v(ii,id,1,ir) - (((
     &              d(ii,jd,1,1,ir)*u(ii,id,1,ir))
     &            + d(ii,jd,2,1,ir)*u(ii,id,2,ir))
     &            + d(ii,jd,3,1,ir)*u(ii,id,3,ir))
               v(ii,id,2,ir) = v(ii,id,2,ir) - (((
     &              d(ii,jd,1,2,ir)*u(ii,id,1,ir))
     &            + d(ii,jd,2,2,ir)*u(ii,id,2,ir))
     &            + d(ii,jd,3,2,ir)*u(ii,id,3,ir))
               v(ii,id,3,ir) = v(ii,id,3,ir) - (((
     &              d(ii,jd,1,3,ir)*u(ii,id,1,ir))
     &            + d(ii,jd,2,3,ir)*u(ii,id,2,ir))
     &            + d(ii,jd,3,3,ir)*u(ii,id,3,ir))
            end do
         end do
      end do

      call rotate(u,nd,nr,nt,-1)
      call rotate(v,nd,nr,nt,-1)

      call comm3s(v,nr,nt,3)

      end
*dk oprxuc
      subroutine oprxuc(v,u,a,ndo,nd,nr,nt)
 
c     This routine computes v = v - A*u, where A is a tensor operator.
 
      include 'pcom.h'
      real a(7,(nt+1)**2,ndo,3,3,3*(nr+1))
      real v((nt+1)**2,nd,3,nr+1)
      real u((nt+1)**2,nd,3,nr+1)
      common /offset/ m(8)
 
      call nuledge(v,nd,nr,nt,3)
 
      call rotate(u,nd,nr,nt,1)
      call rotate(v,nd,nr,nt,1)
 
      nt1 = nt + 1

      do ir=1,nr+1
 
         k1 = 1
         k2 = 3
         if(ir .eq.    1) k1 = 2
         if(ir .eq. nr+1) k2 = 2
 
         do k=k1,k2
 
            jr =  ir + k - 2
            kr = (ir - 1)*3 + k
 
!DIR$ NOINTERCHANGE
            do i=1,3
               do id=1,nd
                  jd = min(ndo, id)
                  do ii=1,(nt+1)**2
                     v(ii,id,1,ir) = v(ii,id,1,ir) - (((((((
     &                 a(1,ii,jd,i,1,kr)*u(ii    ,id,i,jr))
     &               + a(2,ii,jd,i,1,kr)*u(ii+1  ,id,i,jr))
     &               + a(3,ii,jd,i,1,kr)*u(ii+nt1,id,i,jr))
     &               + a(4,ii,jd,i,1,kr)*u(ii+nt ,id,i,jr))
     &               + a(5,ii,jd,i,1,kr)*u(ii-1  ,id,i,jr))
     &               + a(6,ii,jd,i,1,kr)*u(ii-nt1,id,i,jr))
     &               + a(7,ii,jd,i,1,kr)*u(ii-nt ,id,i,jr))
                     v(ii,id,2,ir) = v(ii,id,2,ir) - (((((((
     &                 a(1,ii,jd,i,2,kr)*u(ii    ,id,i,jr))
     &               + a(2,ii,jd,i,2,kr)*u(ii+1  ,id,i,jr))
     &               + a(3,ii,jd,i,2,kr)*u(ii+nt1,id,i,jr))
     &               + a(4,ii,jd,i,2,kr)*u(ii+nt ,id,i,jr))
     &               + a(5,ii,jd,i,2,kr)*u(ii-1  ,id,i,jr))
     &               + a(6,ii,jd,i,2,kr)*u(ii-nt1,id,i,jr))
     &               + a(7,ii,jd,i,2,kr)*u(ii-nt ,id,i,jr))
                     v(ii,id,3,ir) = v(ii,id,3,ir) - (((((((
     &                 a(1,ii,jd,i,3,kr)*u(ii    ,id,i,jr))
     &               + a(2,ii,jd,i,3,kr)*u(ii+1  ,id,i,jr))
     &               + a(3,ii,jd,i,3,kr)*u(ii+nt1,id,i,jr))
     &               + a(4,ii,jd,i,3,kr)*u(ii+nt ,id,i,jr))
     &               + a(5,ii,jd,i,3,kr)*u(ii-1  ,id,i,jr))
     &               + a(6,ii,jd,i,3,kr)*u(ii-nt1,id,i,jr))
     &               + a(7,ii,jd,i,3,kr)*u(ii-nt ,id,i,jr))
                  end do
               end do
            end do
 
         end do
 
      end do
 
      call rotate(u,nd,nr,nt,-1)
      call rotate(v,nd,nr,nt,-1)
 
      if(lvproc .eq. 0) then
         call edgadd0(v,3,nr,nt,1)
      else
         call comm3s(v,nr,nt,3)
      end if
 
      end

*dk bdoffset
      block data bdoffset
  
      common /offset/ m(8)

      data m/0,0,1,1,0,-1,-1,0/
   
      end
